/*	Interface to Roy Wood's TE32K library.

	94/01/02 aih cut/copy/paste work on desk scrap, not private scrap
	93/12/19 aih created by copying and modifying TextEditLib.c */

#include <limits.h>
#include "DrawLib.h"
#include "MemoryLib.h"
#include "RectangleLib.h"
#include "ScrapLib.h"
#include "TextEdit32KLib.h"

static void LPointToLongPoint(const LPointType *lpt, LongPoint *longPt)
{
	longPt->h = lpt->h;
	longPt->v = lpt->v;
}

static void LongPointToLPoint(const LongPoint *longPt, LPointType *lpt)
{
	lpt->h = longPt->h;
	lpt->v = longPt->v;
}

static void LRectToLongRect(const LRectType *lrect, LongRect *longRect)
{
	require(LRectValid(lrect));
	longRect->top = lrect->top;
	longRect->left = lrect->left;
	longRect->bottom = lrect->bottom;
	longRect->right = lrect->right;
}

static void LongRectToLRect(const LongRect *longRect, LRectType *lrect)
{
	lrect->top = longRect->top;
	lrect->left = longRect->left;
	lrect->bottom = longRect->bottom;
	lrect->right = longRect->right;
	ensure(LRectValid(lrect));
}

void TE32KDoGetPoint(long offset, LPointType *lpt, TE32KHandle te)
{
	LongPoint longPt;
	
	TE32KGetPoint(offset, &longPt, te);
	LongPointToLPoint(&longPt, lpt);
}

long TE32KDoGetOffset(const LPointType *lpt, TE32KHandle te)
{
	LongPoint longPt;
	
	LPointToLongPoint(lpt, &longPt);
	return(TE32KGetOffset(&longPt, te));
}

void TE32KView(Rect *view, TE32KHandle te)
{
	LongRect lview;
	
	lview = (**te).viewRect;
	LongRectToRect(&lview, view);
	ensure(RectValid(view));
}

void TE32KViewSet(const Rect *view, TE32KHandle te)
{
	LongRect lview;
	
	require(RectValid(view));
	RectToLongRect((Rect *) view, &lview);
	(**te).viewRect = lview;
}

void TE32KDest(LRectType *dest, TE32KHandle te)
{
	LongRect longDest;
	
	longDest = (**te).destRect;
	LongRectToLRect(&longDest, dest);
	ensure(LRectValid(dest));
}

void TE32KDestSet(const LRectType *dest, TE32KHandle te)
{
	LongRect longDest;

	require(LRectValid(dest));
	LRectToLongRect(dest, &longDest);
	(**te).destRect = longDest;
}

void TE32KStyleContinuous(short *mode, TextStyle *style, TE32KHandle te)
{
	short height, ascent;
	TE32KStyle(style, &height, &ascent, te);
}

void TE32KStyle(TextStyle *style, short *height, short *ascent, TE32KHandle te)
{
	memclr(style, sizeof(TextStyle));
	style->tsFont = (**te).txFont;
	style->tsFace = (**te).txFace;
	style->tsSize = (**te).txSize;
	*height = (**te).lineHeight;
	*ascent = (**te).fontAscent;
}

void TE32KStyleSet(short mode, const TextStyle *style, Boolean redraw,
	TE32KHandle te)
{
	Rect view;
	LongRect lview;
	FontInfo info;
	GrafPtr port = NULL;
	TextState svtext;
	TextStyle apply;
	short height, ascent;
	
	GetPort(&port);
	SetPort((**te).inPort);
	TE32KStyle(&apply, &height, &ascent, te);
	if (mode & doFont)
		apply.tsFont = style->tsFont;
	if (mode & doFace) {
		if (mode & doToggle)
			apply.tsFace ^= style->tsFace;
		else
			apply.tsFace = style->tsFace;
	}
	if (mode & addSize) {
		apply.tsSize += style->tsSize;
		if (apply.tsSize < 1)
			apply.tsSize = 1;
	}
	else if (mode & doSize)
		apply.tsSize = style->tsSize;
	TE32KSetFontStuff(apply.tsFont, apply.tsFace, (**te).txMode, apply.tsSize, te);
	if (redraw) {
		TE32KCalText(te);
		TE32KView(&view, te);
		EraseRect(&view);
		lview = (**te).viewRect;
		TE32KUpdate(&lview, te);
	}
	SetPort(port);
}

StScrpHandle TE32KStyleScrap(TE32KHandle te)
{
	TextStyle style;
	short height, ascent;
	StScrpHandle scrp = NULL;
	
	TE32KStyle(&style, &height, &ascent, te);
	scrp = HandleBeginClear(sizeof(short) + sizeof(ScrpSTElement));
	(**scrp).scrpNStyles = 1;
	(**scrp).scrpStyleTab[0].scrpStartChar = (**te).selStart;
	(**scrp).scrpStyleTab[0].scrpHeight = height;
	(**scrp).scrpStyleTab[0].scrpAscent = ascent;
	(**scrp).scrpStyleTab[0].scrpFont = style.tsFont;
	(**scrp).scrpStyleTab[0].scrpFace = style.tsFace;
	(**scrp).scrpStyleTab[0].scrpSize = style.tsSize;
	ensure(HandleValidSize(scrp, sizeof(short) + sizeof(ScrpSTElement)));
	return(scrp);
}

void TE32KStyleScrapSet(StScrpHandle scrp, Boolean redraw, TE32KHandle te)
{
	TextStyle style;
	
	if (scrp && (**scrp).scrpNStyles > 0) {
		style.tsFont = (**scrp).scrpStyleTab[0].scrpFont;
		style.tsFace = (**scrp).scrpStyleTab[0].scrpFace;
		style.tsSize = (**scrp).scrpStyleTab[0].scrpSize;
		style.tsColor = (**scrp).scrpStyleTab[0].scrpColor;
		TE32KStyleSet(doAll, &style, redraw, te);
	}
}

void TE32KDoCut(TE32KHandle te)
{
	TE32KDoCopy(te);
	TE32KDelete(te);
}

void TE32KDoCopy(TE32KHandle te)
{
	SignedByte state;
	
	ZeroScrap();
	state = HandleLock((**te).hText);
	PutScrap((**te).selEnd - (**te).selStart, 'TEXT',
		*(**te).hText + (**te).selStart);
	HandleRestore((**te).hText, state);
}

void TE32KDoPaste(TE32KHandle te)
{
	SignedByte state;
	Handle scrap;

	TE32KDelete(te);
	scrap = ScrapHandle();
	if (scrap) {
		state = HandleLock(scrap);
		TE32KInsert(*scrap + ScrapOffset('TEXT'), ScrapLength('TEXT'), te);
		HandleRestore(scrap, state);
	}
}

void TE32KReplaceText(Handle text, TE32KHandle te)
{
	HandleEnd((**te).hText);
	(**te).hText = text;
	(**te).selStart = (**te).selEnd = (**te).teLength = HandleSize(text);
}

void TE32KDoActivate(Boolean active, TE32KHandle te)
{
	if (active)
		TE32KActivate(te);
	else
		TE32KDeactivate(te);
}

void TE32KDoUpdate(const Rect *view, TE32KHandle te)
{
	LongRect lview;
	
	RectToLongRect((Rect *) view, &lview);
	TE32KUpdate(&lview, te);
}

long TE32KGetHeight(long start, long end, TE32KHandle te)
{
	if (start == end && end < (**te).nLines + 1)
		end++;
	return((**te).lineHeight * (end - start));
}
